package org.astakhova.data.impl;

import org.astakhova.data.INode;
import org.astakhova.data.IArrow;

/**
 * Makes a description of an arrow
 * @author Astakhova Julia A.
 */
public class Arrow implements IArrow{
	private Node mySource;
	private Node myDest;
	private String myLabel;
	
	/**
	 * Generates an arrow
	 * @param source a node the arrow comes from
	 * @param dest a node the arrows gets to
	 */
	Arrow(Node source, Node dest) {
		mySource = source;
		myDest = dest;
	}
	
	/**
	 * Sets a new values for destination-node of the arrow.
	 * @param source a new value for destination-node
	 */
	public void setDest(INode dest) {
		myDest.removeInArrow(this);
		myDest = (Node) dest;
		myDest.addInArrow(this);
	}
	
	/**
	 * Gets a source-node of the arrow
	 * @return a node the arrow comes from
	 */
	public Node getSource() {
		return mySource;
	}
	
	/**
	 * Gets a destination-node of the arrow
	 * @return a node the arrow gets to
	 */
	public Node getDest() {
		return myDest;
	}
	
	/**
	 * Removes the arrow from the diagram
	 */
	void removeArrow() {
		if (mySource instanceof ConditionNode)
			mySource.setOutArrow(mySource.indexOfOutArrow(this), 
					null);
		else
			mySource.removeOutArrow(this);
		myDest.removeInArrow(this);
	}
	
	public String toString() {
		String s = "The arrow description:\n";
		s += "the arrow dest node:\n";
		s += "the dest text: " + myDest.getText() + "\n";
		s += "the dest coordinates: " + myDest.getX() + " " + 
			myDest.getY() +"\n";
		return s;
	}
	
	public void setLabel(String s) {
		myLabel = s;
	}
	
	public String getLabel() {
		return myLabel;
	}
}
