package org.astakhova.view;

import org.astakhova.data.*;
import org.astakhova.view.param.SColor;

import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Color;

/**
 * Makes a description of a conditional block
 * @author Astakhova Julia A.
 */
public class ConditionBlock extends Block {
	private static final long serialVersionUID = 1;
	private Color myLeftColor = SColor.usual();
	private Color myRightColor = SColor.usual();
	
	/**
	 * Creates a new conditional block
	 * @param n conditional node
	 */
	public ConditionBlock(IConditionNode n) {
		super(n);
	}
	
	protected void paintBlock(Graphics g) {
		Rectangle r = getParameters();
		int h = r.height/2 + r.y;
		int w = r.width/2 + r.x;
		g.setColor(myLeftColor);
		g.drawLine(r.x, h, w, r.y);
		g.drawLine(w, r.y + r.height, r.x, h);
		g.setColor(myRightColor);
		g.drawLine(w, r.y, r.x + r.width, h);
		g.drawLine(r.x + r.width, h, w, r.y + r.height);
	}
	
	public void setTextArea() {
		Rectangle r = getParameters();
		int x = r.x + r.width/10, y = r.y + r.height/5;
		int w = r.width - (x - r.x)*2;
		int h = r.height - (y - r.y)*2;
		setTextArea(new Rectangle(x, y, w, h));
	}
	
	/**
	 * Selects a certain part of the block with the color
	 * @param c color to select
	 * @param b branch of the block connected to the one's part
	 */
	public void setSelected(Color c, Boolean b) {
		if (b == true)
			myLeftColor = c;
		else
			myRightColor = c;
	}
	
	public void setSelected(Color c) {
		myLeftColor = c;
		myRightColor = c;
		super.setSelected(c);
	}
	
	/**
	 * Checks if the point is inside the part of the block
	 * @param p checking point
	 * @param b branch of the block connected to the one's part
	 * @return if the point is inside the part of the block
	 */
	public boolean isInside(Point p, Boolean b) {
		Rectangle r = getParameters();
		return b ?
				(p.x-5 >= r.x 
				&& p.x-5 <= (r.x + r.width/2)
				&& p.y-27 >= r.y
				&& p.y-27 <= (r.y + r.height)) :
					
				(p.x-5 >= (r.x + r.width/2)
				&& p.x-5 <= (r.x + r.width)
				&& p.y-27 >= r.y
				&& p.y-27 <= (r.y + r.height));
	}
}
