package org.astakhova.view;

import java.awt.Graphics;
import java.awt.Point;
import javax.swing.*;
import java.awt.Color;
import org.astakhova.data.*;
import org.astakhova.view.param.SColor;

/**
 * Makes a description of lines. Line is a future link (graphical arrow view)
 * that has only source, but not the target node. So it hasn't arrow ending
 *  and finishes in the cursor point
 * @author Astakhova Julia A.
 *
 */
public class Line extends JComponent{
	static final long serialVersionUID = 1;
	
	private int mySrcX, mySrcY;
	private int myTgtX, myTgtY;
	private INode mySource;
	
	private int mySrcRight, mySrcLeft, mySrcTop, mySrcBottom;
	private int myTgtRight, myTgtLeft, myTgtTop, myTgtBottom;
	
	private Color myColor = SColor.usual();
	private Point[] mySpike = new Point[3];
	
	{
		for (int i = 0; i < 3; i++)
			mySpike[i] = new Point();
	}
	
	/**
	 * Creates a new line
	 */
	public Line() {
		setSize((Math.abs(mySrcX - myTgtX) + 15)*100, 
				(Math.abs(mySrcY - myTgtY) + 15)*100);
	}
	
	public void paintComponent(Graphics g) {
		super.paintComponent(g);

		if( mySrcBottom < myTgtTop ) {
			mySrcX = (mySrcRight + mySrcLeft) / 2;
		    mySrcY = mySrcBottom;

		    myTgtX = (myTgtRight + myTgtLeft) / 2;
		    myTgtY = myTgtTop;
		} else
			if( mySrcTop > myTgtBottom ) {
				mySrcX = (mySrcRight + mySrcLeft) / 2;
				mySrcY = mySrcTop;

				myTgtX = (myTgtRight + myTgtLeft) / 2;
				myTgtY = myTgtBottom;
			} else
				if( mySrcRight < myTgtLeft) {
					mySrcX = mySrcRight;
					mySrcY = (mySrcBottom + mySrcTop) / 2;

					myTgtX = myTgtLeft;
					myTgtY = (myTgtBottom + myTgtTop) / 2;
				} else
					if( mySrcLeft > myTgtRight ) {
						mySrcX = mySrcLeft;
						mySrcY = (mySrcBottom + mySrcTop) / 2;

						myTgtX = myTgtRight;
						myTgtY = (myTgtBottom + myTgtTop) / 2;
					} else
						return;
		
		g.setColor(myColor);
		g.drawLine(mySrcX, mySrcY, myTgtX, myTgtY);
		mySpike[0].x = myTgtX;
		mySpike[0].y = myTgtY;
		
		getSpikePoints();
		g.drawLine(mySpike[0].x, mySpike[0].y, mySpike[1].x, mySpike[1].y);
		g.drawLine(mySpike[2].x, mySpike[2].y, mySpike[0].x, mySpike[0].y);
	}
	
	private void getSpikePoints() {
		  double  rotate = Math.PI / 9; 
		  double  size = 12.0;   
		  int     lineWidth = mySrcX - myTgtX;
		  int     lineHeight = mySrcY - myTgtY;
		  double  angle;
		  
		  if( lineWidth != 0 ) {
			  angle = Math.atan( (double)lineHeight / (double) lineWidth);
			  if( lineWidth < 0 )
				  angle += Math.PI;
		  } else
			  angle = (lineHeight > 0) ? Math.PI / 2.0
		                                 : -Math.PI / 2.0;
		                                 
		  double  angle1 = angle + rotate;
		  double  angle2 = angle - rotate;
		  mySpike[1].x = myTgtX + (int) Math.floor(
				  size*Math.cos(angle1) + 0.5);
		  mySpike[1].y = myTgtY + (int) Math.floor(
				  size*Math.sin(angle1) + 0.5);
		  mySpike[2].x = myTgtX + (int) Math.floor( 
				  size*Math.cos(angle2) + 0.5);
		  mySpike[2].y = myTgtY + (int) Math.floor( 
				  size*Math.sin(angle2) + 0.5);
	}
	
	/**
	 * Sets a new line parameters
	 * @param sl source left x-coordinate
	 * @param sr source right x-coordinate
	 * @param st source top y-coordinate
	 * @param sb source bottom y-coordinate
	 * @param tl target left x-coordinate
	 * @param tr target right x-coordinate
	 * @param tt target top y-coordinate
	 * @param tb target bottom y-coordinate
	 */
	public void setParameters(int sl, int sr, int st, int sb, 
			int tl, int tr, int tt, int tb) {
		mySrcLeft = sl;
		mySrcRight = sr;
		mySrcTop = st;
		mySrcBottom = sb;
		
		myTgtLeft = tl;
		myTgtRight = tr;
		myTgtTop = tt;
		myTgtBottom = tb;
	}
	
	/**
	 * Checks if the point is over the line
	 * @param p point
	 * @return true if the point is over the line
	 */
	public boolean isInside(Point p) {
		double minDistance = 12.0;
		p.x = p.x - 5;
		p.y = p.y - 27;

		if(mySrcX == myTgtX) {
			if( p.y < Math.min(mySrcY, myTgtY)  ||  
					  Math.max(mySrcY, myTgtY) < p.y )
				  return false;
			return (Math.abs(p.x - mySrcX) < minDistance);
		} else
			if(mySrcY == myTgtY) {
				if(p.x < Math.min(mySrcX, myTgtX)  
						  ||  Math.max(mySrcX, myTgtX) < p.x)
					  return false;
				return (Math.abs(p.y - mySrcY) < minDistance);
			}
		if(mySrcX < myTgtX) {
			if(p.x < mySrcX  ||  myTgtX < p.x)
				return false;
		} else {
			if(p.x < myTgtX  ||  mySrcX < p.x)
				return false;
		}
		if(mySrcY < myTgtY) {
			if(p.y < mySrcY  ||  myTgtY < p.y)
				return false;
		} else {
			if(p.y < myTgtY  ||  mySrcY < p.y)
				return false;
		}
		
		double[] prms = getLinePrms( mySrcX, mySrcY, myTgtX, myTgtY);
		double a = prms[0];
		double b = prms[1];
		double c = prms[2];

		// (b*x - a*y + c2 = 0):
		double  c2 = -(b*p.x - a*p.y);

	
		// (b*xt - a*yt + c2 = 0)
		// (a*xt + b*yt + c  = 0):
		double  yt = (a*c2 - b*c) / (a*a + b*b);
		double  xt = -(b*yt + c) / a;

	
		double  dx = xt - p.x;
		double  dy = yt - p.y;
		double  z = Math.sqrt( dx*dx + dy*dy);

		return  (z < minDistance);
	}
	
	private double[] getLinePrms( int x1, int y1, int x2, int y2) {
	  int  dy = y2 - y1;
	  int  dx = x2 - x1;
	  double[] prms = new double[3]; // a, b, c
	  
	  if( dx == 0 ) { 
		  prms[1] = 0.0;
		  prms[0] = 1.0;
	  } else
		  if( dy == 0 )  { 
			  prms[1] = 1.0;
			  prms[0] = 0.0;
		  } else {
			  prms[1] = 1.0;
			  prms[0] = prms[1] * ((double)( -dy) / (double)( dx));
	  }
	  prms[2] = -(prms[0]*x1 + prms[1]*y1);
	  return prms;
	}
	
	/**
	 * Selects the line with the color 
	 * @param c color to select
	 */
	public void setSelected(Color c) {
		myColor = c;
	}
	
	public Color getColor() {
		return myColor;
	}
	
	public void setTargetPoint(int x, int y) {
		myTgtLeft = x;
		myTgtRight = x;
		myTgtTop = y;
		myTgtBottom = y;
	}
	
	public void setSourcePoint(int x, int y) {
		mySrcLeft = x;
		mySrcRight = x;
		mySrcTop = y;
		mySrcBottom = y;
	}
	
	public void setNode(INode n) {
		mySource = n;
	}
	
	public INode getNode() {
		return mySource;
	}
}
