package org.astakhova.view.listener;

import java.awt.event.MouseEvent;
import org.astakhova.data.IArrow;
import org.astakhova.data.IBeginNode;
import org.astakhova.view.AbstractLink;
import org.astakhova.view.Block;
import org.astakhova.view.ErrorForm;
import org.astakhova.view.Line;
import org.astakhova.view.param.SColor;

/**
 * Makes a description of mouse listener for delete action button
 * @author Astakhova Julia A.
 */
public class DeleteMouseL extends MouseL {	
	public void mouseClicked(MouseEvent e) {
		try {
			//removing a block ...
			java.util.List<Block> ucblock = getForm().getUnderCursor().getBlockList();
			if (ucblock.size() > 0) {
				Block b = ucblock.get(0);
				if (b.getNode() instanceof IBeginNode)
					getForm().getOwner().checkButtonPossibility();
				getForm().getDiagram().removeNode(b.getNode());
				getForm().getContentPane().remove(b);
				
				// ... with all its in and out links
				for (Line a : getForm().getUnderCursor().getLineList()) 
					getForm().getContentPane().remove(a);
				
				getForm().getUnderCursor().clear();
				getForm().repaint();
				return;
			}
			
			//removing a selected link
			java.util.List<Line> ucline = getForm().getUnderCursor().getLineList();
			if (ucline.size() > 0) {
				Line l = ucline.get(0);
				getForm().getDiagram().removeArrow(((AbstractLink) l).getArrow());
				getForm().getContentPane().remove(l);
				
				getForm().getUnderCursor().clear();
				getForm().repaint();
				return;
			}
		} finally {
			getForm().getOwner().checkButtonPossibility();
			getForm().fixChanges();
		}
		new ErrorForm(getForm().getOwner(), null, 
				"No element was selected.", false);
	}

	public void mouseMoved(MouseEvent e) {
		//selecting blocks and links that will be removed if a click is performed now
		
		boolean wasSelected = false;
		getForm().getUnderCursor().clear();
		for (java.awt.Component b : getForm().getContentPane().getComponents()) {
			if (b instanceof Block) {
				if (!wasSelected && ((Block) b).isInside(e.getPoint())) {
					getForm().getUnderCursor().addBlock((Block) b);
					wasSelected = true;
				} else
					((Block) b).setSelected(SColor.usual());
			}
			if (b instanceof AbstractLink) {
				if (!wasSelected && 
						((AbstractLink) b).isInside(e.getPoint())) {
					getForm().getUnderCursor().addLink((AbstractLink) b);
					wasSelected = true;
				} else
					((AbstractLink) b).setSelected(SColor.usual());
			}
		}
			
		java.util.List<Block> ucblock = getForm().getUnderCursor().getBlockList();
		if (ucblock.size() > 0) {
			Block bf = ucblock.get(0);
			for (java.awt.Component b : getForm().getContentPane().getComponents()) {
				if (b instanceof AbstractLink) {
					IArrow ai = ((AbstractLink) b).getArrow();
					if (ai.getSource() == bf.getNode() ||
							ai.getDest() == bf.getNode()) {
						getForm().getUnderCursor().addLink((AbstractLink) b);
	
					}
				}
			}
		}
		getForm().getUnderCursor().setSelected(SColor.selecting());
		getForm().repaint();
	}
	
	public boolean isPossible() {
		return (getForm().getDiagram().getNodes().size() != 0);
	}
}
